
//Parameters for the circle (radius, number of items)
int numDim = 12;
int radius = 80;
int plotHeight = 40;

int highestQuantity = -1;
float gap = 0.0;
PImage bg;

int state = 0;
String yearStr = "2005";

int curHour = 0;

PFont fontTitle;
PFont fontTxt;
PFont fontMiddle;

ArrayList daysAL; // list of all the days
ArrayList itemsTypeAL; //list of all the items type which are on all the entries
Day currDay; //the day which is displayed on the software

void setup(){
  
  size(1024,768);
  smooth();
  
  fontTitle = loadFont("Miso-45.vlw");
  fontTxt = loadFont("Miso-16.vlw");
  fontMiddle = loadFont("Miso-20.vlw");
  
  bg = loadImage("bg.jpg");
  
  daysAL = new ArrayList();
  itemsTypeAL = new ArrayList();
  
  processDatas("splxmldata.txt");
  
  //Print result (for debug) + get the highest value + sort ItemsType
  for(int i=0; i<daysAL.size(); i++){
    Day d = (Day) daysAL.get(i);
    
    println("-" + d.day);
    for(int j=0; j<d.hours.length; j++){
      println("---" + d.hours[j].h);
      d.hours[j].sortItemsType();
      for(int k=0; k<d.hours[j].itemsQuantity.size(); k++){
        ItemsQuantity iq = (ItemsQuantity) d.hours[j].itemsQuantity.get(k);
        println("-----" + iq.type + " : " + iq.quantity);
        if(iq.quantity > highestQuantity){
          highestQuantity = iq.quantity;
        }
      }
    }
  }
  
  println("Highest quantity: " + highestQuantity);

  //Get the first day
  currDay = (Day) daysAL.get(state);
}


void draw(){
  
  image(bg,0,0);
  noStroke();
  
  drawLegends();
  
  fill(#111111);
  textAlign(CENTER);
  textFont(fontTitle,45);
  text(yearStr, width/2, height/2+14);
  
  translate(width/2,height/2);
  rotate(radians(150));
  
  //put the different hours of the day in a circle shape
  for(int i=0; i<numDim; i++){
    float theta = radians(i*360.0/numDim);
    float x = cos(theta)*radius;
    float y = sin(theta)*radius;
    pushMatrix();
    translate(x,y);
    rotate(theta);
    float posX = 0.0;
    
    //get all item types for the hour
    for(int j=0; j<currDay.hours[i].itemsQuantity.size(); j++){
      ItemsQuantity iq = (ItemsQuantity) currDay.hours[i].itemsQuantity.get(j);
      float plotWidth = map(iq.quantity,20,highestQuantity,1, 200);
      //just draw a kind of "highlight" to the current hour
      if(curHour == i){
        fill(255,100);
        rect(posX-5, -(plotHeight/2)-5, plotWidth+10, plotHeight+10);
      }
      //draw the plot which reprensent a specific item(iq) type at a specific hour(curHour)
      fill(getColor(iq.type));
      rect(posX,-(plotHeight/2),plotWidth,plotHeight);
      posX += (plotWidth + gap);
    }
    popMatrix();
  }
}


//Compute the legends
void drawLegends(){
  
  int offY = 600;
  int offX = 15;
  
  textFont(fontMiddle, 20);
  textAlign(LEFT);
  
  fill(#111111);
  text(currDay.hours[curHour].h,offX, offY);
  stroke(#111111);
  strokeWeight(1);
  line(offX, offY+10, offX+180, offY+10);
  offY += 30;
  
  
  
  noStroke();
  
  textFont(fontTxt, 16);
  
  fill(getColor("Book"));
  rect(offX, offY-5, 10, 10);
  fill(#111111);
  textAlign(LEFT);
  text("Books", offX+15, offY+6);
  textAlign(RIGHT);
  text(currDay.hours[curHour].getQuantityItem("Book"),offX+130,offY+6);
  offY += 20;
  
  fill(getColor("Video (VHS)"));
  rect(offX, offY-5, 10, 10);
  fill(#111111);
  textAlign(LEFT);
  text("Video (VHS)", offX+15, offY+6);
  textAlign(RIGHT);
  text(currDay.hours[curHour].getQuantityItem("Video (VHS)"),offX+130,offY+6);
  offY += 20;
  
  fill(getColor("Audio Tape"));
  rect(offX, offY-5, 10, 10);
  fill(#111111);
  textAlign(LEFT);
  text("Audio Tape", offX+15, offY+6);
  textAlign(RIGHT);
  text(currDay.hours[curHour].getQuantityItem("Audio Tape"),offX+130,offY+6);
  offY += 20;
  
  fill(getColor("CD"));
  rect(offX, offY-5, 10, 10);
  fill(#111111);
  textAlign(LEFT);
  text("CD", offX+15, offY+6);
  textAlign(RIGHT);
  text(currDay.hours[curHour].getQuantityItem("CD"),offX+130,offY+6);
  offY += 20;
  
  fill(getColor("CD-ROM"));
  rect(offX, offY-5, 10, 10);
  fill(#111111);
  textAlign(LEFT);
  text("CD-ROM", offX+15, offY+6);
  textAlign(RIGHT);
  text(currDay.hours[curHour].getQuantityItem("CD-ROM"),offX+130,offY+6);
  offY += 20;
  
  fill(getColor("DVD"));
  rect(offX, offY-5, 10, 10);
  fill(#111111);
  textAlign(LEFT);
  text("DVD", offX+15, offY+6);
  textAlign(RIGHT);
  text(currDay.hours[curHour].getQuantityItem("DVD"),offX+130,offY+6);
  offY += 20;
  
}

//Process all the datas...build the right software hierarchy. in order to use those datas directly on the draw, without calculating anything
void processDatas(String fileName){
  
  println("Loading and processing datas...");
  
  String[] lines = loadStrings(fileName);
  ArrayList tmpDates = new ArrayList();
  
  for(int i=0; i<lines.length; i++){
    String[] splittedL = split(lines[i], ',');
    
    //Add a new date
    if(!dateExists(splittedL[0], tmpDates)){
      tmpDates.add(splittedL[0]);
    }
    
    //Create the itemtype of the entry
    String itemTypeStr = checkItemType(splittedL[2]);
    if(!itemTypeStr.equals("NULL")){
      ItemType item = new ItemType(splittedL[0], splittedL[1], itemTypeStr);
      itemsTypeAL.add(item);
    }
  }
  
  //For each date on tmpDates, create the Date object. When the date is created, it will create aswell the 12 hours and the itemsquantity for each of those hours
  for(int i=0; i<tmpDates.size(); i++){
    String dayStr = (String) tmpDates.get(i);
    Day dayTmp = new Day(dayStr, itemsTypeAL);
    daysAL.add(dayTmp);
  }
}


//Check if the date already exists
boolean dateExists(String date, ArrayList tmpDates){
  for(int i=0; i<tmpDates.size(); i++){
    String dayTmp = (String) tmpDates.get(i);
    if(date.equals(dayTmp)){
      return true;
    }
  }
  return false;
}


//Convert general type into his color
color getColor(String t){
  
  if(t.equals("Audio Tape")){
    return color(#f2cd26);
  }
  else if(t.equals("Book")){
    return color(#ee452b);
  }
  else if(t.equals("CD")){
    return color(#cbe071);
  }
  else if(t.equals("CD-ROM")){
    return color(#8aeece);
  }
  else if(t.equals("DVD")){
    return color(#5fcada);
  }
  else if(t.equals("Video (VHS)")){
    return color(#f68c12);
  }
  
  return color(255,255,255);
}


//Convert itemtype into general type
String checkItemType(String def){
  if(def.equals("accas") || def.equals("arcas") || def.equals("bccas") || def.equals("jccas") || def.equals("jrcas")){
    return "Audio Tape";
  }
  else if(def.equals("acbk") || def.equals("arbk") || def.equals("bcbk") || def.equals("drbk") || def.equals("jcbk") || def.equals("jrbk")){
    return "Book";
  }
  else if(def.equals("accd") || def.equals("arcd") || def.equals("jccd") || def.equals("jrcd")){
    return "CD";
  }
  else if(def.equals("accdrom") || def.equals("arcdrom") || def.equals("bccdrom") || def.equals("drcdrom") || def.equals("jccdrom")){
    return "CD-ROM";
  }
  else if(def.equals("acdvd") || def.equals("ardvd") || def.equals("bcdvd") || def.equals("jcdvd") || def.equals("jrdvd")){
    return "DVD";
  }
  else if(def.equals("acvhs") || def.equals("alvhs") || def.equals("arvhs") || def.equals("bcvhs") || def.equals("blvhs") || def.equals("jcvhs") || def.equals("jlvhs") || def.equals("jrvhs") || def.equals("xrvhs") || 
  def.equals("scmed") || def.equals("acvid") || def.equals("arvid")){
    return "Video (VHS)";
  }
  
  return "NULL";
}


void keyPressed(){
  if(key == CODED){
    if(keyCode == RIGHT){
      int tmp = Integer.parseInt(yearStr);
      if(tmp < 2010){
        yearStr = Integer.toString(tmp + 1);
        state++;
        currDay = (Day) daysAL.get(state);
      }
      else{
        yearStr = "2005";
        state = 0;
        currDay = (Day) daysAL.get(state);
      }
    }
    else if(keyCode == LEFT){
      int tmp = Integer.parseInt(yearStr);
      if(tmp > 2005){
        yearStr = Integer.toString(tmp - 1);
        state--;
        currDay = (Day) daysAL.get(state);
      }
      else{
        yearStr = "2010";        
        state = 5;
        currDay = (Day) daysAL.get(state);
      }
    }
    else if(keyCode == UP){
      curHour++;
      if(curHour > 11){
        curHour = 0;
      }
    }
    else if(keyCode == DOWN){
      curHour--;
      if(curHour < 0){
        curHour = 11;
      }
    }
  }
}
